from __future__ import print_function
from builtins import range




# Input Format is:
#   a,b c,d
# where
#   a and b are the number of nts away from a UMI for the first/second UMI
#   c and d are the number of UMIs that are a and b nts away from the first and second UMI respectively
# and always:
#   a <= b
# and sometimes:
#   c and d can be either an integer or a range.
#
# Output Format is as follows:
#   Given:
#       "a,b c,d"
#   Output:
#       (a,b,c,d), (b,a,d,c)
meta_format = """
0,0 0,0
0,1 0,1
0,1 0,2-4
0,2 0,1
0,2 0,2-8
0,3 0,1
0,3 0,2-11
0,4 0,1-8

1,1 1,1
1,1 1,2-4
1,1 2-4,2-4

1,2 1,1
1,2 1,2-8
1,2 2-4,2-8

2,2 1,1
2,2 1,2-8
2,2 2-8,2-8


USE THIS TO SORT!!!
http://stackoverflow.com/a/673892
http://stackoverflow.com/a/673892
http://stackoverflow.com/a/673892
http://stackoverflow.com/a/673892
http://stackoverflow.com/a/673892
http://stackoverflow.com/a/673892
http://stackoverflow.com/a/673892


"""

#thing = set()
thing = []

for line in meta_format.split('\n'):
    line = line.strip()
    if line == '':
        continue
    if line[0] == '#':
        continue

    dists, num_umis = line.split()

    dist1, dist2 = dists.split(',')
    dist1 = int(dist1)
    dist2 = int(dist2)

    n_umis1, n_umis2 = num_umis.split(',')

    if '-' in n_umis1:
        s,e = n_umis1.split('-')
        s = int(s)
        e = int(e)
        n_umis1 = list(range(s, e+1))
    else:
        n_umis1 = [int(n_umis1)]

    if '-' in n_umis2:
        s,e = n_umis2.split('-')
        s = int(s)
        e = int(e)
        n_umis2 = list(range(s, e+1))
    else:
        n_umis2 = [int(n_umis2)]


    for n_umi1 in n_umis1:
        for n_umi2 in n_umis2:
            # thing.add((dist1, dist2, n_umi1, n_umi2))
            # thing.add((dist2, dist1, n_umi2, n_umi1))
            thing.append((dist1, dist2, n_umi1, n_umi2))
            thing.append((dist2, dist1, n_umi2, n_umi1))

print(thing)




#!/usr/bin/env python


from __future__ import print_function
from builtins import object
class Thing(object):
    __slots__ = 'foo bar'.split()

    def put(self, something, other):
        self.foo = something
        self.bar = other

import functools

t = Thing()


f_put = functools.partial(t.put, 'astring')

f_put('another_str')
print(t)
print(t.foo)
print(t.bar)

# d = {}
# f_put2 = functools.partial(d[], 'astring')



#!/usr/bin/env python

from __future__ import print_function
import sys

"""
Usage:
        ~/code/uora/dupliganger/dupliganger/dupliganger/pipeline.py 4 outdir ACTTGATG_R1.3300 ACTTGATG_R2.3300
        ~/dupliganger/dupliganger/dupliganger/pipeline.py 4 outdir ACTTGATG_R1.3300 ACTTGATG_R2.3300

"""
# Usage:

num_threads = sys.argv[1]
outdir = sys.argv[2]
base = sys.argv[3]
base1 = sys.argv[4]
base2 = sys.argv[5]

obase1 = outdir + '/' + base1
obase2 = outdir + '/' + base2

print()
print("export PATH=/Users/jason/Library/Python/2.7/bin/:$PATH")
print()

cmd1 = "time dupliganger -o {} remove-umi -t {} --kit bioo {}.fq.gz {}.fq.gz".format(outdir, num_threads, base1, base2)
cmd2 = "time dupliganger -o {} remove-adapter {}.rmumi.fq {}.rmumi.fq".format(outdir, obase1, obase2)
cmd3 = "time dupliganger -o {} qtrim -t {} {}.rmumi.rmadapt.fq {}.rmumi.rmadapt.fq".format(outdir, num_threads, obase1, obase2)
# cmd4 = "dupliganger -o {} annotate-qtrim -t 2 {}.rmumi.rmadapt.fq {}.rmumi.rmadapt.fq {}.rmumi.rmadapt.qtrim.fq {}.rmumi.rmadapt.qtrim.fq".format(outdir, obase1, obase2, obase1, obase2)
cmd4 = "time dupliganger -o {} annotate-qtrim {}.rmumi.rmadapt.qtrim.fq {}.rmumi.rmadapt.qtrim.fq".format(outdir, obase1, obase2)
cmd5 = "mkdir -p {}/alignments".format(outdir)
cmd6 = "time gsnap -D ~/seq/gsnap/assemblies/zebrafish/Danio_rerio.GRCz10.dna.toplevel -d Danio_rerio.GRCz10.dna.toplevel -k 15 -B 5 -m 0.1 -t 30 -s ~/seq/gsnap/assemblies/zebrafish/Danio_rerio.GRCz10.dna.toplevel/Danio_rerio.GRCz10.dna.toplevel.maps/splicesitesfile.iit -A sam --split-output={}/alignments/{} -Q {}.rmumi.rmadapt.qtrim.anno.fq {}.rmumi.rmadapt.qtrim.anno.fq".format(outdir, base, obase1, obase2)

cmd = "{} && {} && {} && {} && {} && {} && echo && (echo Success && echo) || (echo Failure && echo)".format(cmd1, cmd2, cmd3, cmd4, cmd5, cmd6)
print("\t\t\t##### no-compress version #####")
print("  --- Commands ---")
print("1 - {}".format(cmd1))
print("2 - {}".format(cmd2))
print("3 - {}".format(cmd3))
print("4 - {}".format(cmd4))
print("5 - {}".format(cmd5))
print("6 - {}".format(cmd6))
print("  --- Copy-Paste ---")
print(cmd)




"""

Basically, generate all possible combinations of 8nts.  For each sequence,
figure out closest UMIs, and count them.  For example, a sequence may have no
UMIs closer then 2 nts; that seq may be 2 nts away from 3 UMIs, and 3nts away
from 10 UMIs, and 4nts away from 44 UMIs, and so on...  Only count / report
that that sequence is 2 nts away from 3 UMIs.



include
Format:
    u1,u2   u1 is the closest away, and it has 

u1 exact, u2 exact                                          --> ACCEPTED                                        0,0 0,0
u1 exact, u2 is 1nt away from 1 UMI                         --> ACCEPTED                                        0,1 0,1
u1 exact, u2 is 1nt away from 2-4 UMIs                      --> ACCEPT and ITERATE over all combos              0,1 0,2-4
u1 exact, u2 is 2nt away from 1 UMI                         --> ACCEPTED                                        0,2 0,1
u1 exact, u2 is 2nt away from 2-8 UMIs                      --> ACCEPT and ITERATE over all combos              0,2 0,2-8
u1 exact, u2 is 3nt away from 1 UMI                         --> NOT SURE                                        0,3 0,1
u1 exact, u2 is 3nt away from 2-11 UMIs                     --> NOT SURE                                        0,3 0,2-11
u1 exact, u2 is 4nt away from 1-8 UMIs                      --> REJECT                                          0,4 0,1-8

u1 is 1nt away from 1 UMI, u2 is 1nt away from 1 UMI        --> ACCEPTED                                        1,1 1,1
u1 is 1nt away from 1 UMI, u2 is 1nt away from 2-4 UMIs     --> ACCEPT and ITERATE over all combos              1,1 1,2-4
u1 is 1nt away from 2-4 UMIs, u2 is 1nt away from 2-4 UMIs  --> ACCEPT and ITERATE over all combos              1,1 2-4,2-4

u1 is 1nt away from 1 UMI, u2 is 2nt away from 1 UMI        --> ACCEPTED                                        1,2 1,1
u1 is 1nt away from 1 UMI, u2 is 2nt away from 2-8 UMIs     --> ACCEPT and ITERATE over all combos              1,2 1,2-8
u1 is 1nt away from 2-4 UMIs, u2 is 2nt away from 2-8 UMIs  --> ACCEPT and ITERATE over all combos              1,2 2-4,2-8

u1 is 2nt away from 1 UMI, u2 is 2nt away from 1 UMI        --> ACCEPTED (probably)                             2,2 1,1
u1 is 2nt away from 1 UMI, u2 is 2nt away from 2-8 UMIs     --> ACCEPT and ITERATE over all combos              2,2 1,2-8
u1 is 2nt away from 2-8 UMIs, u2 is 2nt away from 2-8 UMIs  --> ACCEPT and ITERATE over all combos              2,2 2-8,2-8

Reject the case where none of the UMIs is at least 2nt away.



USE THIS TO SORT!!!
http://stackoverflow.com/a/673892
http://stackoverflow.com/a/673892
http://stackoverflow.com/a/673892
http://stackoverflow.com/a/673892
http://stackoverflow.com/a/673892
http://stackoverflow.com/a/673892
http://stackoverflow.com/a/673892



0,0 0,0
0,1 0,1
0,1 0,2-4
0,2 0,1
0,2 0,2-8
0,3 0,1
0,3 0,2-11
0,4 0,1-8

1,1 1,1
1,1 1,2-4
1,1 2-4,2-4

1,2 1,1
1,2 1,2-8
1,2 2-4,2-8

2,2 1,1
2,2 1,2-8
2,2 2-8,2-8









raw report
{0: 96, 1: {1: 2038, 2: 110, 3: 14, 4: 1}, 2: {1: 15280, 2: 3060, 3: 474, 4: 104, 5: 25, 6: 4, 7: 1, 8: 1}, 3: {1: 10252, 2: 11496, 3: 8625, 4: 4774, 5: 2166, 6: 860, 7: 325, 8: 72, 9: 20, 10: 6, 11: 1}, 4: {1: 8, 2: 52, 3: 162, 4: 324, 5: 541, 6: 691, 7: 792, 8: 780, 9: 728, 10: 593, 11: 424, 12: 263, 13: 179, 14: 94, 15: 45, 16: 35, 17: 8, 18: 3, 19: 3, 20: 1, 24: 1}, 5: {24: 1, 18: 1, 28: 1, 21: 1}, 6: {}, 7: {}, 8: {}}



96 exact matches

distance == 0
	Number of sequences that have ** 1 ** closest UMIs: 96

distance == 1
	Number of sequences that have ** 1 ** closest UMIs: 2038
	Number of sequences that have ** 2 ** closest UMIs: 110
	Number of sequences that have ** 3 ** closest UMIs: 14
	Number of sequences that have ** 4 ** closest UMIs: 1

distance == 2
	Number of sequences that have ** 1 ** closest UMIs: 15280
	Number of sequences that have ** 2 ** closest UMIs: 3060
	Number of sequences that have ** 3 ** closest UMIs: 474
	Number of sequences that have ** 4 ** closest UMIs: 104
	Number of sequences that have ** 5 ** closest UMIs: 25
	Number of sequences that have ** 6 ** closest UMIs: 4
	Number of sequences that have ** 7 ** closest UMIs: 1
	Number of sequences that have ** 8 ** closest UMIs: 1

distance == 3
	Number of sequences that have ** 1 ** closest UMIs: 10252
	Number of sequences that have ** 2 ** closest UMIs: 11496
	Number of sequences that have ** 3 ** closest UMIs: 8625
	Number of sequences that have ** 4 ** closest UMIs: 4774
	Number of sequences that have ** 5 ** closest UMIs: 2166
	Number of sequences that have ** 6 ** closest UMIs: 860
	Number of sequences that have ** 7 ** closest UMIs: 325
	Number of sequences that have ** 8 ** closest UMIs: 72
	Number of sequences that have ** 9 ** closest UMIs: 20
	Number of sequences that have ** 10 ** closest UMIs: 6
	Number of sequences that have ** 11 ** closest UMIs: 1

distance == 4
	Number of sequences that have ** 1 ** closest UMIs: 8
	Number of sequences that have ** 2 ** closest UMIs: 52
	Number of sequences that have ** 3 ** closest UMIs: 162
	Number of sequences that have ** 4 ** closest UMIs: 324
	Number of sequences that have ** 5 ** closest UMIs: 541
	Number of sequences that have ** 6 ** closest UMIs: 691
	Number of sequences that have ** 7 ** closest UMIs: 792
	Number of sequences that have ** 8 ** closest UMIs: 780
	Number of sequences that have ** 9 ** closest UMIs: 728
	Number of sequences that have ** 10 ** closest UMIs: 593
	Number of sequences that have ** 11 ** closest UMIs: 424
	Number of sequences that have ** 12 ** closest UMIs: 263
	Number of sequences that have ** 13 ** closest UMIs: 179
	Number of sequences that have ** 14 ** closest UMIs: 94
	Number of sequences that have ** 15 ** closest UMIs: 45
	Number of sequences that have ** 16 ** closest UMIs: 35
	Number of sequences that have ** 17 ** closest UMIs: 8
	Number of sequences that have ** 18 ** closest UMIs: 3
	Number of sequences that have ** 19 ** closest UMIs: 3
	Number of sequences that have ** 20 ** closest UMIs: 1
	Number of sequences that have ** 24 ** closest UMIs: 1

distance == 5
	Number of sequences that have ** 18 ** closest UMIs: 1
	Number of sequences that have ** 21 ** closest UMIs: 1
	Number of sequences that have ** 24 ** closest UMIs: 1
	Number of sequences that have ** 28 ** closest UMIs: 1

distance == 6

distance == 7

distance == 8

"""
from __future__ import print_function





















import distance

UMIS_BIOO = ['AACGCCAT', 'AAGGTACG', 'AATTCCGG', 'ACACAGAG', 'ACACTCAG',
        'ACACTGTG', 'ACAGGACA', 'ACCTGTAG', 'ACGAAGGT', 'ACGACTTG', 'ACGTCAAC',
        'ACGTCATG', 'ACTGTCAG', 'ACTGTGAC', 'AGACACTC', 'AGAGGAGA', 'AGCATCGT',
        'AGCATGGA', 'AGCTACCA', 'AGCTCTAG', 'AGGACAAC', 'AGGACATG', 'AGGTTGCT',
        'AGTCGAGA', 'AGTGCTGT', 'ATAAGCGG', 'ATCCATGG', 'ATCGAACC', 'ATCGCGTA',
        'ATCGTTGG', 'CAACGATC', 'CAACGTTG', 'CAACTGGT', 'CAAGTCGT', 'CACACACA',
        'CAGTACTG', 'CATCAGCA', 'CATCGTTC', 'CCAAGGTT', 'CCTAGCTT', 'CGATTACG',
        'CGCCTATT', 'CGTTCCAT', 'CGTTGGAT', 'CTACGTTC', 'CTACTCGT', 'CTAGAGGA',
        'CTAGGAAG', 'CTAGGTAC', 'CTCAGTCT', 'CTGACTGA', 'CTGAGTGT', 'CTGATGTG',
        'CTGTTCAC', 'CTTCGTTG', 'GAACAGGT', 'GAAGACCA', 'GAAGTGCA', 'GACATGAG',
        'GAGAAGAG', 'GAGAAGTC', 'GATCCTAG', 'GATGTCGT', 'GCCGATAT', 'GCCGATTA',
        'GCGGTATT', 'GGAATTGG', 'GGATAACG', 'GGCCTAAT', 'GGCGTATT', 'GTCTTGTC',
        'GTGATGAG', 'GTGATGTC', 'GTGTACTG', 'GTGTAGTC', 'GTTCACCT', 'GTTCTGCT',
        'GTTGTCGA', 'TACGAACC', 'TAGCAAGG', 'TAGCTAGC', 'TAGGTTCG', 'TATAGCGC',
        'TCAGGACT', 'TCCACATC', 'TCGACTTC', 'TCGTAGGT', 'TCGTCATC', 'TGAGACTC',
        'TGAGAGTG', 'TGAGTGAG', 'TGCTTGGA', 'TGGAGTAG', 'TGTGTGTG', 'TTCGCCTA',
        'TTCGTTCG']


report = {}
# Set frequently accessed ones...
report[0] = 0
report[1] = {}
report[2] = {}
report[3] = {}
report[4] = {}
report[5] = {}
report[6] = {}
report[7] = {}
report[8] = {}


for i1 in 'ACGT':
# for i1 in 'A':
    print()
    print()
    print()
    print("i1 = {}".format(i1))
    for i2 in 'ACGT':
    # for i2 in 'A':
        print("i2 = {}".format(i2))
        for i3 in 'ACGT':
            for i4 in 'ACGT':
                for i5 in 'ACGT':
                    for i6 in 'ACGT':
                        for i7 in 'ACGT':
                            for i8 in 'ACGT':
                                seq_umi = "{}{}{}{}{}{}{}{}".format(
                                        i1, i2, i3, i4, i5, i6, i7, i8)
                                max_dist = 9999
                                d = 9999
                                found_umis = []
                                for umi in UMIS_BIOO:
                                    d = distance.hamming(umi, seq_umi)
                                    if d == max_dist:
                                        found_umis.append(umi)
                                    elif d < max_dist:
                                        # reset found_umis
                                        found_umis = [umi]
                                        max_dist = d
                                    if max_dist == 0:
                                        break
                                
                                # report
                                if max_dist == 0:
                                    report[0] += 1
                                else:
                                    # report[5][3] += 1
                                    # report.setdefault(max_dist, {})
                                    report[max_dist].setdefault(len(found_umis), 0)
                                    report[max_dist][len(found_umis)] += 1


print()
print()
print("raw report")
print(report)
print()

print("{} exact matches".format(report[0]))
for dist in [1, 2, 3, 4, 5, 6, 7, 8]:
    print()
    print("distance == {}".format(dist))
    for num_found_umis in sorted(report[dist].keys()):
        print("\tNumber of sequences that have ** {} ** closest UMIs: {}".format( num_found_umis, report[dist][num_found_umis]  ))





"""
Usage:
    dupliganger prep [options] <input.fastq>
    dupliganger prep [options] <in1.fastq> <in2.fastq>
    dupliganger prep [options] <input.bam>

Global Options:
    -h, --help
    -v, --verbose         Be verbose
    -k KIT, --kit KIT     The kit used [default: bioo].
    -o OUT_DIR            Place results in directory OUT_DIR.
    --compress            Compress (gzip) output files.
    -t N, --threads N     Number of threads to run with [default: 1].

Pipeline Options:
    --retain-intermediates      Do not delete intermediate files.
    --compress-intermediates    Compress intermediate files.
    --pipeline-names            Retain pipeline filenames.

remove-umi Options:
    --force-paired        Do not autodetect whether paired-end vs single-end,
                          instead force paired-end. Helpful fail safe if you
                          believe you have paired end data.

remove-adapter Options:
    -C S, --cutadapt S  Pass 'S' as arguments to Cutadapt. Don't forget to put quotes
                        around S if passing in more than one argument.
                        [default: '-n 3 -O 1 -m 30']
    -1 A, --adapter1    First (Illumina) adapter [default: GATCGGAAGAGCACACG]
    -2 A, --adapter2    Second (Illumina) adapter [default: AGATCGGAAGAGCGTCG]

qtrim Options:
    -T S, --trimmomatic S   Pass 'S' as arguments to Trimmomatic. Don't forget to put quotes
                            around S if passing in more than one argument.
                            [default: 'LEADING:10 TRAILING:10 SLIDINGWINDOW:5:10 MINLEN:30']
    -p P, --phred P         Set P to 33 for phred33 encoded files, or 64 for
                            phred 64 encoded FASTQ files [default: 33].

annotate-qtrim Options:
    -l TRIMLOG            Optionally specify trimlog.

-----

New Options Ideas:
    --restart       Restart the prep pipeline.
    --tmpdir        Temporary directory.


























































    dupliganger remove-umi [options] <input.bam>
Options:
    -h, --help
    -v, --verbose         Be verbose
    -k KIT, --kit KIT     The kit used [default: bioo].
    -o OUT_DIR            Place results in directory OUT_DIR.
    --compress            Compress (gzip) output files.
    --force-paired        Do not autodetect whether paired-end vs single-end,
                          instead force paired-end. Helpful fail safe if you
                          believe you have paired end data.
    -t N, --threads N     EXPERIMENTAL: If pigz is installed and --threads is
                          specified, output FASTQ files will be compressed with
                          pigz -p <n>; otherwise, they will be left
                          uncompressed (as it simply takes too long to compress
                          with just gzip).



    dupliganger remove-adapter [options] <in1.fastq> <in2.fastq>
Options:
    -C S, --cutadapt S  Pass 'S' as arguments to Cutadapt. Don't forget to put quotes
                        around S if passing in more than one argument.
                        [default: '-n 3 -O 1 -m 30']
    -1 A, --adapter1    First (Illumina) adapter [default: GATCGGAAGAGCACACG]
    -2 A, --adapter2    Second (Illumina) adapter [default: AGATCGGAAGAGCGTCG]



    dupliganger qtrim [options] <in1.fastq> <in2.fastq>
Options:
    -h, --help
    -v, --verbose           Be verbose.
    -o OUT_DIR              Place results in directory OUT_DIR.
    --compress              Compress (gzip) output files.
    -t N, --threads N       Number of threads to run with [default: 1].
    -T S, --trimmomatic S   Pass 'S' as arguments to Trimmomatic. Don't forget to put quotes
                            around S if passing in more than one argument.
                            [default: 'LEADING:10 TRAILING:10 SLIDINGWINDOW:5:10 MINLEN:30']
    -p P, --phred P         Set P to 33 for phred33 encoded files, or 64 for
                            phred 64 encoded FASTQ files [default: 33].





    dupliganger annotate-qtrim [options] <in1.fastq> <in2.fastq>
Options:
    -l TRIMLOG            Optionally specify trimlog.

"""



"""Annotates read names with UMIs and clips inline UMIs if needed.

Usage:
    dupliganger remove-umi [options] <input.fastq>
    dupliganger remove-umi [options] <in1.fastq> <in2.fastq>
    dupliganger remove-umi [options] <input.bam>


Note:
    Dupliganger supports (and autodetects) input FASTQ files that are gzipped.

Note:
    If passing a paired-end BAM file, it needs to be sorted by read name
    (if not sorted, dupliganger will exit out when it detects mismatching
    adjacent records).

Options:
    -h, --help
    -v, --verbose         Be verbose
    -k KIT, --kit KIT     The kit used [default: bioo].
    -o OUT_DIR            Place results in directory OUT_DIR.
    --compress            Compress (gzip) output files.
    --force-paired        Do not autodetect whether paired-end vs single-end,
                          instead force paired-end. Helpful fail safe if you
                          believe you have paired end data.
    -t N, --threads N     EXPERIMENTAL: If pigz is installed and --threads is
                          specified, output FASTQ files will be compressed with
                          pigz -p <n>; otherwise, they will be left
                          uncompressed (as it simply takes too long to compress
                          with just gzip).
"""
from __future__ import print_function




import docopt

def main():
    args = docopt(__doc__)
    print(args)
    exit()

